#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2012 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
"""
Virginia report views.
"""

from zope.browserpage.viewpagetemplatefile import ViewPageTemplateFile
from zope.i18n.interfaces.locales import ICollator
from zope.security.proxy import removeSecurityProxy
from zope.traversing.browser.absoluteurl import absoluteURL

from schooltool.basicperson.interfaces import IDemographics
from schooltool.cando.interfaces import ISkillsGradebook
from schooltool.cando.skill import querySkillScoreSystem
from schooltool.course.interfaces import ISection
from schooltool.requirement.scoresystem import UNSCORED
from schooltool.skin import flourish


class VASectionReportView(flourish.page.Page):

    content_template = ViewPageTemplateFile('templates/va_section_report.pt')
    container_class = 'container extra-wide-container'

    @property
    def section(self):
        gradebook = removeSecurityProxy(self.context)
        return ISection(gradebook)

    @property
    def subtitle(self):
        return self.section.title

    def next_url(self):
        return absoluteURL(self.context, self.request)

    @property
    def scoresystem(self):
        return querySkillScoreSystem()

    @property
    def passing_score(self):
        scoresystem = self.scoresystem
        if self.filter_field_id in self.request:
            rating_score = self.request[self.filter_field_id]
        else:
            rating_score = scoresystem._minPassingScore
        return rating_score

    @property
    def passing_scores(self):
        result = []
        scoresystem = self.scoresystem
        if self.filter_field_id in self.request:
            rating_score = self.request[self.filter_field_id]
        else:
            rating_score = scoresystem._minPassingScore
        for score_info in scoresystem.scores:
            label = score_info[0]
            # XXX: use real scoresystem values here instead of int()
            if int(label) >= int(rating_score):
                result.append(label)
        return '/'.join(reversed(result))

    @property
    def filter_field_id(self):
        return 'minimum_rating_score'

    @property
    def scores(self):
        result = []
        scoresystem = self.scoresystem
        for score_info in scoresystem.scores:
            if self.filter_field_id in self.request:
                selected = score_info[0] == self.request[self.filter_field_id]
            else:
                selected = scoresystem.isPassingScore(score_info[0])
            result.append({
                    'id': score_info[0],
                    'title': score_info[1],
                    'selected': selected,
                    })
        return result
            
    def update(self):
        course_titles = []
        course_government_ids = []
        for course in self.section.courses:
            course_titles.append(course.title)
            course_government_ids.append(course.government_id or '')
        self.course = {
            'title': ', '.join(course_titles),
            'government_id': ', '.join(course_government_ids),
            }
        instructors = []
        for person in self.section.instructors:
            instructors.append(person.title)
        self.instructors = '<br />'.join(instructors)
        rows = []
        collator = ICollator(self.request.locale)
        worksheets = self.context.__parent__.__parent__
        students_count = len(self.section.members)
        for student in sorted(self.section.members,
                              key=lambda x:x.title,
                              cmp=collator.cmp):
            demographics = IDemographics(student)
            info = {
                'student': student.title,
                'state_report_id': demographics.get('state_report_id', ''),
                }
            info.update(self.getStudentSkillsData(student, worksheets))
            rows.append(info)
        self.rows = rows
        attainment_labels = {100: '90-100%',
                             90: '80-89%',
                             80: '70-79%',
                             70: '60-69%',
                             60: '<= 59%'}
        rows_attainment = {100: 0,
                           90: 0,
                           80: 0,
                           70: 0,
                           60: 0}
        rows_attainment80 = {u'80% or above': 0,
                             u'Below 80%': 0}
        for row in self.rows:
            rows_attainment[row['rows_attainment']] += 1
            rows_attainment80[row['rows_attainment80']] += 1
        self.rows_attainment = []
        for key in sorted(rows_attainment, reverse=True):
            info = {"scale": attainment_labels[key],
                    "student_count": rows_attainment[key]}
            try:
                p = rows_attainment[key] / float(students_count)
            except (ZeroDivisionError,):
                p = 0
            info["percentage"] = self.formatPercentage(p)
            self.rows_attainment.append(info)
        self.rows_attainment80 = []
        for key in sorted(rows_attainment80):
            info = {"scale": key,
                    "student_count": rows_attainment80[key]}
            try:
                p = rows_attainment80[key] / float(students_count)
            except (ZeroDivisionError,):
                p = 0
            info["percentage"] = self.formatPercentage(p)
            self.rows_attainment80.append(info)

    def getStudentSkillsData(self, student, worksheets):
        required_total = 0
        required_evaluated = 0
        required_competent = 0
        all_total = 0
        all_evaluated = 0
        all_competent = 0
        for worksheet in worksheets.values():
            gradebook = ISkillsGradebook(worksheet)
            for skill in gradebook.activities:
                all_total += 1
                score = gradebook.getScore(student, skill)
                evaluated = score is not None and score.value is not UNSCORED
                # XXX: use real scoresystem values here instead of int()
                competent = evaluated and \
                            int(score.value) >= int(self.passing_score)
                if evaluated:
                    all_evaluated += 1
                if competent:
                    all_competent += 1
                if skill.required:
                    required_total += 1
                    if evaluated:
                        required_evaluated += 1
                    if competent:
                        required_competent += 1
        try:
            required_rating = required_competent / float(required_evaluated)
        except (ZeroDivisionError,):
            required_rating = 0
        try:
            required_completed = required_competent / float(required_total)
        except (ZeroDivisionError,):
            required_completed = 0
        try:
            all_rating = all_competent / float(all_evaluated)
        except (ZeroDivisionError,):
            all_rating = 0
        if required_completed >= 90:
            rows_attainment = 100
        elif required_completed >= 80:
            rows_attainment = 90
        elif required_completed >= 70:
            rows_attainment = 80
        elif required_completed >= 60:
            rows_attainment = 70
        else:
            rows_attainment = 60
        if required_completed >= 80:
            rows_attainment80 = u'80% or above'
        else:
            rows_attainment80 = u'Below 80%'
        return {
            'required_total': required_total,
            'required_evaluated': required_evaluated,
            'required_competent': required_competent,
            'required_rating': self.formatPercentage(required_rating),
            'required_completed': self.formatPercentage(required_completed),
            'all_total': all_total,
            'all_evaluated': all_evaluated,
            'all_competent': all_competent,
            'all_rating': self.formatPercentage(all_rating),
            'rows_attainment': rows_attainment,
            'rows_attainment80': rows_attainment80,
            }

    def formatPercentage(self, number):
        return '%.0f%%' % (number * 100)
